import asyncio
import concurrent
import copy
import inspect
import threading
import time
import uuid
from typing import Any, Callable, Dict, List, Optional, cast

from gentrace.api_client import ApiClient
from gentrace.apis.tags.v1_api import V1Api
from gentrace.apis.tags.v2_api import V2Api
from gentrace.configuration import Configuration
from gentrace.model.run_v2 import RunV2
from gentrace.providers.context import Context
from gentrace.providers.pipeline import Pipeline
from gentrace.providers.step_run import StepRun
from gentrace.providers.utils import (
    GENTRACE_CONFIG_STATE,
    from_date_string,
    get_test_counter,
    is_openai_v1,
    run_post_background,
    to_date_string,
)

_pipeline_run_loop = None
_pipeline_tasks = []


# https://stackoverflow.com/a/63110035/1057411
def fire_and_forget(coro):
    global _pipeline_run_loop, _pipeline_tasks
    if _pipeline_run_loop is None:
        _pipeline_run_loop = asyncio.new_event_loop()
        threading.Thread(target=_pipeline_run_loop.run_forever, daemon=True).start()
    if inspect.iscoroutine(coro):
        task = asyncio.run_coroutine_threadsafe(coro, _pipeline_run_loop)
        _pipeline_tasks.append(task)


def flush():
    global _pipeline_tasks
    if _pipeline_tasks:
        # Wait for all tasks to complete
        concurrent.futures.wait(_pipeline_tasks)
        _pipeline_tasks.clear()


def get_run(
        run_id: str,
) -> RunV2:
    """
    Retrieves a run for a given run ID from the Gentrace API.

    Args:
        run_id (str): The run ID to retrieve.

    Raises:
        ValueError: If the SDK is not initialized. Call init() first.

    Returns:
        RunV2: The run.
    """

    config = GENTRACE_CONFIG_STATE["global_gentrace_config"]
    if not config:
        raise ValueError("Gentrace API key not initialized. Call init() first.")

    api_client = ApiClient(configuration=config)
    api = V2Api(api_client=api_client)

    response = api.v2_runs_id_get({"id": run_id})
    run = response.body
    return run


class PipelineRun:
    def __init__(
            self, pipeline, id: Optional[str] = None, context: Optional[Context] = None
    ):
        self.pipeline: Pipeline = pipeline
        self.pipeline_run_id: str = id or str(uuid.uuid4())
        self.step_runs: List[StepRun] = []
        self.context: Context = context or {}
        self.error: Optional[str] = None

    def get_id(self):
        return self.pipeline_run_id
    
    def set_error(self, error: str | None):
        self.error = error

    def get_error(self):
        return self.error

    def get_pipeline(self):
        return self.pipeline

    def get_openai(self, asynchronous=False):
        if "openai" in self.pipeline.pipeline_handlers:

            if is_openai_v1():
                from gentrace.providers.llms.openai_v1 import (
                    GentraceAsyncOpenAI,
                    GentraceSyncOpenAI,
                )

                if asynchronous:
                    openai_async_handler = GentraceAsyncOpenAI(**self.pipeline.openai_config,
                                                               gentrace_config=self.pipeline.config,
                                                               pipeline=self, pipeline_run=self)

                    from openai import AsyncOpenAI

                    typed_cloned_handler = cast(AsyncOpenAI, openai_async_handler)
                    return typed_cloned_handler
                else:
                    openai_handler = GentraceSyncOpenAI(**self.pipeline.openai_config,
                                                        gentrace_config=self.pipeline.config,
                                                        pipeline=self, pipeline_run=self)

                    from openai import OpenAI

                    typed_cloned_handler = cast(OpenAI, openai_handler)
                    return typed_cloned_handler
            else:
                handler = self.pipeline.pipeline_handlers.get("openai")
                cloned_handler = copy.deepcopy(handler)

                from .llms.openai_v0 import annotate_pipeline_handler

                annotated_handler = annotate_pipeline_handler(
                    cloned_handler, self.pipeline.openai_config, self
                )

                import openai

                # TODO: Could not find an easy way to create a union type with openai and
                # OpenAIPipelineHandler, so we just use openai.
                typed_cloned_handler = cast(openai, annotated_handler)

                return typed_cloned_handler
        else:
            raise ValueError(
                "Did not find OpenAI handler. Did you call setup() on the pipeline?"
            )

    def get_pinecone(self):
        if "pinecone" in self.pipeline.pipeline_handlers:
            handler = self.pipeline.pipeline_handlers.get("pinecone")
            cloned_handler = copy.deepcopy(handler)
            cloned_handler.set_pipeline_run(self)
            import pinecone

            return cast(pinecone, cloned_handler)
        else:
            raise ValueError(
                "Did not find Pinecone handler. Did you call setup() on the pipeline?"
            )

    def add_step_run(self, step_run: StepRun):
        self.step_runs.append(step_run)

    async def ameasure(self, func: Callable[..., Any], **kwargs):
        """
        Asynchronously measures the execution time of a function and logs the result as a `StepRun`.
        Also logs additional information about the function invocation.

        Parameters:
        func (Callable[..., Any]): The asynchronous function whose execution time is to be measured.
        **kwargs: Arbitrary keyword arguments. These are passed directly to the function.
                  If a "step_info" argument is included, it should be a dictionary containing
                  additional metadata about the function invocation. Supported keys are "provider",
                  "invocation", and "model_params". The "step_info" argument is not passed to the
                  function.

        Returns:
        The return value of the function invocation.

        Raises:
        Any exceptions raised by the function will be propagated.

        Example:
        async def add(x, y):
            return x + y

        await measure(add, x=1, y=2, step_info={"provider": "my_provider", "invocation": "add invocation"})
        """
        input_params = {k: v for k, v in kwargs.items() if k not in ["step_info"]}

        step_info = kwargs.get("step_info", {})

        start_time = time.time()
        error = None
        outputs = {}
        try:
            outputs = await func(**input_params)
        except Exception as e:
            error = e

        end_time = time.time()

        outputs_for_step_run = outputs

        if not isinstance(outputs_for_step_run, dict):
            outputs_for_step_run = {"value": outputs_for_step_run}

        elapsed_time = int((end_time - start_time) * 1000)

        self.add_step_run(
            StepRun(
                step_info.get("provider", "undeclared"),
                step_info.get("invocation", "undeclared"),
                elapsed_time,
                to_date_string(start_time),
                to_date_string(end_time),
                input_params,
                step_info.get("model_params", {}),
                outputs_for_step_run,
                step_info.get("context", {}),
                str(error) if error else None,
            )
        )

        if error:
            raise error

        return outputs

    def measure(self, func: Callable[..., Any], **kwargs):
        """
        Measures the execution time of a function and logs the result as a `StepRun`.
        Also logs additional information about the function invocation.

        Parameters:
        func (Callable[..., Any]): The function whose execution time is to be measured.
        **kwargs: Arbitrary keyword arguments. These are passed directly to the function.
                  If a "step_info" argument is included, it should be a dictionary containing
                  additional metadata about the function invocation. Supported keys are "provider",
                  "invocation", and "model_params". The "step_info" argument is not passed to the
                  function.

        Returns:
        The return value of the function invocation.

        Raises:
        Any exceptions raised by the function will be propagated.

        Example:
        def add(x, y):
            return x + y

        measure(add, x=1, y=2, step_info={"provider": "my_provider", "invocation": "add invocation"})
        """
        input_params = {k: v for k, v in kwargs.items() if k not in ["step_info"]}

        step_info = kwargs.get("step_info", {})

        start_time = time.time()
        outputs = {}
        error = None
        try:
            outputs = func(**input_params)
        except Exception as e:
            error = e

        end_time = time.time()

        outputs_for_step_run = outputs

        if not isinstance(outputs_for_step_run, dict):
            outputs_for_step_run = {"value": outputs_for_step_run}

        elapsed_time = int((end_time - start_time) * 1000)

        self.add_step_run(
            StepRun(
                step_info.get("provider", "undeclared"),
                step_info.get("invocation", "undeclared"),
                elapsed_time,
                to_date_string(start_time),
                to_date_string(end_time),
                input_params,
                step_info.get("model_params", {}),
                outputs_for_step_run,
                step_info.get("context", {}),
                str(error) if error else None,
            )
        )

        if error:
            raise error

        return outputs

    def checkpoint(self, step_info):
        """
        Creates a checkpoint by recording a `StepRun` instance with execution metadata and appending it to `self.step_runs`.
        If there are no prior steps, elapsed time is set to 0 and start and end times are set to the current timestamp.
        If prior steps exist, calculates the elapsed time using the end time of the last `StepRun`.

        Parameters:
        step_info (dict): The information about the step to checkpoint. It should include "inputs" and "outputs".
                    Optionally, it can also include "provider", "invocation" and "modelParams".

        Returns:
        None

        Example:
        checkpoint_step = {
            "provider": "MyProvider",
            "invocation": "doSomething",
            "inputs": {"x": 10, "y": 20},
            "outputs": {"result": 30}
        }

        checkpoint(checkpoint_step)
        """
        last_element = self.step_runs[-1] if self.step_runs else None

        if last_element:
            step_start_time = from_date_string(last_element.end_time)
            end_time_new = time.time()
            elapsed_time = int((end_time_new - step_start_time) * 1000)
            self.step_runs.append(
                StepRun(
                    step_info.get("provider", "undeclared"),
                    step_info.get("invocation", "undeclared"),
                    elapsed_time,
                    to_date_string(step_start_time),
                    to_date_string(end_time_new),
                    step_info.get("inputs", {}),
                    step_info.get("modelParams", {}),
                    step_info.get("outputs", {}),
                    step_info.get("context", {}),
                    step_info.get("error", None),
                )
            )
        else:
            elapsed_time = 0
            start_and_end_time = time.time()
            self.step_runs.append(
                StepRun(
                    step_info.get("provider", "undeclared"),
                    step_info.get("invocation", "undeclared"),
                    elapsed_time,
                    to_date_string(start_and_end_time),
                    to_date_string(start_and_end_time),
                    step_info.get("inputs", {}),
                    step_info.get("modelParams", {}),
                    step_info.get("outputs", {}),
                    step_info.get("context", {}),
                    step_info.get("error", None),
                )
            )

    def get_final_step_runs(self):
        merged_metadata = {}

        step_runs_data = []
        for step_run in self.step_runs:
            # Extract metadata without mutating original contexts
            this_context = copy.deepcopy(self.context)
            this_context_metadata = this_context.get("metadata", {})
            step_run_context = copy.deepcopy(step_run.context)
            step_run_context_metadata = step_run_context.get("metadata", {})

            merged_metadata.update(this_context_metadata)
            merged_metadata.update(step_run_context_metadata)

            this_context.pop("metadata", None)
            step_run_context.pop("metadata", None)

            this_context.pop("previousRunId", None)
            step_run_context.pop("previousRunId", None)

            step_runs_data.append(
                {
                    "providerName": step_run.provider,
                    "invocation": step_run.invocation,
                    "modelParams": step_run.model_params,
                    "inputs": step_run.inputs,
                    "outputs": step_run.outputs,
                    "elapsedTime": step_run.elapsed_time,
                    "startTime": step_run.start_time,
                    "endTime": step_run.end_time,
                    "context": {**this_context, **step_run_context},
                }
            )

        return (
            step_runs_data,
            merged_metadata
        )

    async def asubmit(self) -> Dict:
        if get_test_counter() > 0:
            return {
                "pipelineRunId": self.get_id(),
            }

        configuration = Configuration(host=self.pipeline.config.get("host"))
        configuration.access_token = self.pipeline.config.get("api_key")
        api_client = ApiClient(configuration=configuration)
        v1_api = V1Api(api_client=api_client)

        step_runs_data, merged_metadata = self.get_final_step_runs()

        if len(step_runs_data) == 0:
            return {"pipelineRunId": None}

        try:
            pipeline_post_response = await run_post_background(
                v1_api,
                {
                    "id": self.pipeline_run_id,
                    "slug": self.pipeline.slug,
                    "metadata": merged_metadata,
                    "previousRunId": self.context.get("previousRunId"),
                    "stepRuns": step_runs_data,
                },
            )
            return {
                "pipelineRunId": pipeline_post_response.body.get_item_oapg(
                    "pipelineRunId"
                )
            }
        except Exception as e:
            print(f"Error submitting to Gentrace: {e}")
            return {"pipelineRunId": None}

    def submit(self, wait_for_server=False) -> Dict:
        if get_test_counter() > 0:
            return {
                "pipelineRunId": self.get_id(),
            }

        configuration = Configuration(host=self.pipeline.config.get("host"))
        configuration.access_token = self.pipeline.config.get("api_key")

        api_client = ApiClient(configuration=configuration)
        v1_api = V1Api(api_client=api_client)

        step_runs_data, merged_metadata = self.get_final_step_runs()

        if len(step_runs_data) == 0:
            return {"pipelineRunId": None}
            
        if not wait_for_server:
            fire_and_forget(
                run_post_background(
                    v1_api,
                    {
                        "id": self.pipeline_run_id,
                        "slug": self.pipeline.slug,
                        "metadata": merged_metadata,
                        "previousRunId": self.context.get("previousRunId"),
                        "stepRuns": step_runs_data,
                    },
                )
            )

            return {"pipelineRunId": self.pipeline_run_id}

        if wait_for_server:
            try:
                pipeline_post_response = v1_api.v1_run_post(
                    {
                        "id": self.pipeline_run_id,
                        "slug": self.pipeline.slug,
                        "metadata": merged_metadata,
                        "previousRunId": self.context.get("previousRunId"),
                        "stepRuns": step_runs_data,
                    }
                )
                return {
                    "pipelineRunId": pipeline_post_response.body.get_item_oapg(
                        "pipelineRunId"
                    )
                }

            except Exception as e:
                print(f"Error submitting to Gentrace: {e}")
                return {"pipelineRunId": None}



import typing_extensions

from gentrace.paths import PathValues
from gentrace.apis.paths.v1_run import V1Run
from gentrace.apis.paths.v1_feedback import V1Feedback
from gentrace.apis.paths.v1_test_case import V1TestCase
from gentrace.apis.paths.v1_test_result import V1TestResult
from gentrace.apis.paths.v1_test_result_id import V1TestResultId
from gentrace.apis.paths.v1_test_result_status import V1TestResultStatus
from gentrace.apis.paths.v1_test_result_simple import V1TestResultSimple
from gentrace.apis.paths.v1_test_result_simple_id import V1TestResultSimpleId
from gentrace.apis.paths.v1_pipelines import V1Pipelines
from gentrace.apis.paths.v1_files_upload import V1FilesUpload
from gentrace.apis.paths.v2_datasets import V2Datasets
from gentrace.apis.paths.v2_datasets_id import V2DatasetsId
from gentrace.apis.paths.v2_feedback import V2Feedback
from gentrace.apis.paths.v2_evaluations import V2Evaluations
from gentrace.apis.paths.v2_feedback_id import V2FeedbackId
from gentrace.apis.paths.v2_runs_id import V2RunsId
from gentrace.apis.paths.v2_test_results import V2TestResults
from gentrace.apis.paths.v2_test_cases import V2TestCases
from gentrace.apis.paths.v2_test_cases_id import V2TestCasesId
from gentrace.apis.paths.v2_pipelines import V2Pipelines
from gentrace.apis.paths.v2_folders import V2Folders
from gentrace.apis.paths.v2_folders_id import V2FoldersId
from gentrace.apis.paths.v2_evaluators import V2Evaluators
from gentrace.apis.paths.v2_evaluations_bulk import V2EvaluationsBulk
from gentrace.apis.paths.v3_evaluations import V3Evaluations

PathToApi = typing_extensions.TypedDict(
    'PathToApi',
    {
        PathValues.V1_RUN: V1Run,
        PathValues.V1_FEEDBACK: V1Feedback,
        PathValues.V1_TESTCASE: V1TestCase,
        PathValues.V1_TESTRESULT: V1TestResult,
        PathValues.V1_TESTRESULT_ID: V1TestResultId,
        PathValues.V1_TESTRESULT_STATUS: V1TestResultStatus,
        PathValues.V1_TESTRESULTSIMPLE: V1TestResultSimple,
        PathValues.V1_TESTRESULTSIMPLE_ID: V1TestResultSimpleId,
        PathValues.V1_PIPELINES: V1Pipelines,
        PathValues.V1_FILES_UPLOAD: V1FilesUpload,
        PathValues.V2_DATASETS: V2Datasets,
        PathValues.V2_DATASETS_ID: V2DatasetsId,
        PathValues.V2_FEEDBACK: V2Feedback,
        PathValues.V2_EVALUATIONS: V2Evaluations,
        PathValues.V2_FEEDBACK_ID: V2FeedbackId,
        PathValues.V2_RUNS_ID: V2RunsId,
        PathValues.V2_TESTRESULTS: V2TestResults,
        PathValues.V2_TESTCASES: V2TestCases,
        PathValues.V2_TESTCASES_ID: V2TestCasesId,
        PathValues.V2_PIPELINES: V2Pipelines,
        PathValues.V2_FOLDERS: V2Folders,
        PathValues.V2_FOLDERS_ID: V2FoldersId,
        PathValues.V2_EVALUATORS: V2Evaluators,
        PathValues.V2_EVALUATIONS_BULK: V2EvaluationsBulk,
        PathValues.V3_EVALUATIONS: V3Evaluations,
    }
)

path_to_api = PathToApi(
    {
        PathValues.V1_RUN: V1Run,
        PathValues.V1_FEEDBACK: V1Feedback,
        PathValues.V1_TESTCASE: V1TestCase,
        PathValues.V1_TESTRESULT: V1TestResult,
        PathValues.V1_TESTRESULT_ID: V1TestResultId,
        PathValues.V1_TESTRESULT_STATUS: V1TestResultStatus,
        PathValues.V1_TESTRESULTSIMPLE: V1TestResultSimple,
        PathValues.V1_TESTRESULTSIMPLE_ID: V1TestResultSimpleId,
        PathValues.V1_PIPELINES: V1Pipelines,
        PathValues.V1_FILES_UPLOAD: V1FilesUpload,
        PathValues.V2_DATASETS: V2Datasets,
        PathValues.V2_DATASETS_ID: V2DatasetsId,
        PathValues.V2_FEEDBACK: V2Feedback,
        PathValues.V2_EVALUATIONS: V2Evaluations,
        PathValues.V2_FEEDBACK_ID: V2FeedbackId,
        PathValues.V2_RUNS_ID: V2RunsId,
        PathValues.V2_TESTRESULTS: V2TestResults,
        PathValues.V2_TESTCASES: V2TestCases,
        PathValues.V2_TESTCASES_ID: V2TestCasesId,
        PathValues.V2_PIPELINES: V2Pipelines,
        PathValues.V2_FOLDERS: V2Folders,
        PathValues.V2_FOLDERS_ID: V2FoldersId,
        PathValues.V2_EVALUATORS: V2Evaluators,
        PathValues.V2_EVALUATIONS_BULK: V2EvaluationsBulk,
        PathValues.V3_EVALUATIONS: V3Evaluations,
    }
)



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class EvaluatorV2(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "createdAt",
            "valueType",
            "name",
            "id",
            "runCondition",
            "updatedAt",
            "who",
        }
        
        class properties:
            id = schemas.UUIDSchema
            createdAt = schemas.Float32Schema
            updatedAt = schemas.Float32Schema
            name = schemas.StrSchema
            who = schemas.StrSchema
            valueType = schemas.StrSchema
            runCondition = schemas.StrSchema
        
            @staticmethod
            def archivedAt() -> typing.Type['UnixSecondsNullable']:
                return UnixSecondsNullable
            
            
            class icon(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'icon':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class options(
                schemas.ListBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneTupleMixin
            ):
            
            
                class MetaOapg:
                    items = schemas.AnyTypeSchema
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[list, tuple, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'options':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            aiModel = schemas.StrSchema
            
            
            class pipelineId(
                schemas.UUIDBase,
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                class MetaOapg:
                    format = 'uuid'
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, uuid.UUID, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'pipelineId':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class processorId(
                schemas.UUIDBase,
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                class MetaOapg:
                    format = 'uuid'
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, uuid.UUID, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'processorId':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            organizationId = schemas.UUIDSchema
            templateDescription = schemas.StrSchema
            
            
            class heuristicFn(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'heuristicFn':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            heuristicFnLanguage = schemas.StrSchema
            
            
            class aiPromptFormat(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'aiPromptFormat':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class aiImageUrls(
                schemas.ListSchema
            ):
            
            
                class MetaOapg:
                    items = schemas.StrSchema
            
                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple[typing.Union[MetaOapg.items, str, ]], typing.List[typing.Union[MetaOapg.items, str, ]]],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'aiImageUrls':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )
            
                def __getitem__(self, i: int) -> MetaOapg.items:
                    return super().__getitem__(i)
            
            
            class humanPrompt(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'humanPrompt':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class classifierValuePath(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'classifierValuePath':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class classifierExpectedValuePath(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'classifierExpectedValuePath':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class multiClassOptions(
                schemas.ListSchema
            ):
            
            
                class MetaOapg:
                    items = schemas.StrSchema
            
                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple[typing.Union[MetaOapg.items, str, ]], typing.List[typing.Union[MetaOapg.items, str, ]]],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'multiClassOptions':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )
            
                def __getitem__(self, i: int) -> MetaOapg.items:
                    return super().__getitem__(i)
            prodEvalActive = schemas.BoolSchema
            
            
            class samplingProbability(
                schemas.NumberBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneDecimalMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, decimal.Decimal, int, float, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'samplingProbability':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            __annotations__ = {
                "id": id,
                "createdAt": createdAt,
                "updatedAt": updatedAt,
                "name": name,
                "who": who,
                "valueType": valueType,
                "runCondition": runCondition,
                "archivedAt": archivedAt,
                "icon": icon,
                "options": options,
                "aiModel": aiModel,
                "pipelineId": pipelineId,
                "processorId": processorId,
                "organizationId": organizationId,
                "templateDescription": templateDescription,
                "heuristicFn": heuristicFn,
                "heuristicFnLanguage": heuristicFnLanguage,
                "aiPromptFormat": aiPromptFormat,
                "aiImageUrls": aiImageUrls,
                "humanPrompt": humanPrompt,
                "classifierValuePath": classifierValuePath,
                "classifierExpectedValuePath": classifierExpectedValuePath,
                "multiClassOptions": multiClassOptions,
                "prodEvalActive": prodEvalActive,
                "samplingProbability": samplingProbability,
            }
    
    createdAt: MetaOapg.properties.createdAt
    valueType: MetaOapg.properties.valueType
    name: MetaOapg.properties.name
    id: MetaOapg.properties.id
    runCondition: MetaOapg.properties.runCondition
    updatedAt: MetaOapg.properties.updatedAt
    who: MetaOapg.properties.who
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["createdAt"]) -> MetaOapg.properties.createdAt: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["updatedAt"]) -> MetaOapg.properties.updatedAt: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["who"]) -> MetaOapg.properties.who: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["valueType"]) -> MetaOapg.properties.valueType: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["runCondition"]) -> MetaOapg.properties.runCondition: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["archivedAt"]) -> 'UnixSecondsNullable': ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["icon"]) -> MetaOapg.properties.icon: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["options"]) -> MetaOapg.properties.options: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["aiModel"]) -> MetaOapg.properties.aiModel: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["pipelineId"]) -> MetaOapg.properties.pipelineId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["processorId"]) -> MetaOapg.properties.processorId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["organizationId"]) -> MetaOapg.properties.organizationId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["templateDescription"]) -> MetaOapg.properties.templateDescription: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["heuristicFn"]) -> MetaOapg.properties.heuristicFn: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["heuristicFnLanguage"]) -> MetaOapg.properties.heuristicFnLanguage: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["aiPromptFormat"]) -> MetaOapg.properties.aiPromptFormat: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["aiImageUrls"]) -> MetaOapg.properties.aiImageUrls: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["humanPrompt"]) -> MetaOapg.properties.humanPrompt: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["classifierValuePath"]) -> MetaOapg.properties.classifierValuePath: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["classifierExpectedValuePath"]) -> MetaOapg.properties.classifierExpectedValuePath: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["multiClassOptions"]) -> MetaOapg.properties.multiClassOptions: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["prodEvalActive"]) -> MetaOapg.properties.prodEvalActive: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["samplingProbability"]) -> MetaOapg.properties.samplingProbability: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["id", "createdAt", "updatedAt", "name", "who", "valueType", "runCondition", "archivedAt", "icon", "options", "aiModel", "pipelineId", "processorId", "organizationId", "templateDescription", "heuristicFn", "heuristicFnLanguage", "aiPromptFormat", "aiImageUrls", "humanPrompt", "classifierValuePath", "classifierExpectedValuePath", "multiClassOptions", "prodEvalActive", "samplingProbability", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["createdAt"]) -> MetaOapg.properties.createdAt: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["updatedAt"]) -> MetaOapg.properties.updatedAt: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["who"]) -> MetaOapg.properties.who: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["valueType"]) -> MetaOapg.properties.valueType: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["runCondition"]) -> MetaOapg.properties.runCondition: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["archivedAt"]) -> typing.Union['UnixSecondsNullable', schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["icon"]) -> typing.Union[MetaOapg.properties.icon, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["options"]) -> typing.Union[MetaOapg.properties.options, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["aiModel"]) -> typing.Union[MetaOapg.properties.aiModel, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["pipelineId"]) -> typing.Union[MetaOapg.properties.pipelineId, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["processorId"]) -> typing.Union[MetaOapg.properties.processorId, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["organizationId"]) -> typing.Union[MetaOapg.properties.organizationId, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["templateDescription"]) -> typing.Union[MetaOapg.properties.templateDescription, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["heuristicFn"]) -> typing.Union[MetaOapg.properties.heuristicFn, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["heuristicFnLanguage"]) -> typing.Union[MetaOapg.properties.heuristicFnLanguage, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["aiPromptFormat"]) -> typing.Union[MetaOapg.properties.aiPromptFormat, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["aiImageUrls"]) -> typing.Union[MetaOapg.properties.aiImageUrls, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["humanPrompt"]) -> typing.Union[MetaOapg.properties.humanPrompt, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["classifierValuePath"]) -> typing.Union[MetaOapg.properties.classifierValuePath, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["classifierExpectedValuePath"]) -> typing.Union[MetaOapg.properties.classifierExpectedValuePath, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["multiClassOptions"]) -> typing.Union[MetaOapg.properties.multiClassOptions, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["prodEvalActive"]) -> typing.Union[MetaOapg.properties.prodEvalActive, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["samplingProbability"]) -> typing.Union[MetaOapg.properties.samplingProbability, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["id", "createdAt", "updatedAt", "name", "who", "valueType", "runCondition", "archivedAt", "icon", "options", "aiModel", "pipelineId", "processorId", "organizationId", "templateDescription", "heuristicFn", "heuristicFnLanguage", "aiPromptFormat", "aiImageUrls", "humanPrompt", "classifierValuePath", "classifierExpectedValuePath", "multiClassOptions", "prodEvalActive", "samplingProbability", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        createdAt: typing.Union[MetaOapg.properties.createdAt, decimal.Decimal, int, float, ],
        valueType: typing.Union[MetaOapg.properties.valueType, str, ],
        name: typing.Union[MetaOapg.properties.name, str, ],
        id: typing.Union[MetaOapg.properties.id, str, uuid.UUID, ],
        runCondition: typing.Union[MetaOapg.properties.runCondition, str, ],
        updatedAt: typing.Union[MetaOapg.properties.updatedAt, decimal.Decimal, int, float, ],
        who: typing.Union[MetaOapg.properties.who, str, ],
        archivedAt: typing.Union['UnixSecondsNullable', schemas.Unset] = schemas.unset,
        icon: typing.Union[MetaOapg.properties.icon, None, str, schemas.Unset] = schemas.unset,
        options: typing.Union[MetaOapg.properties.options, list, tuple, None, schemas.Unset] = schemas.unset,
        aiModel: typing.Union[MetaOapg.properties.aiModel, str, schemas.Unset] = schemas.unset,
        pipelineId: typing.Union[MetaOapg.properties.pipelineId, None, str, uuid.UUID, schemas.Unset] = schemas.unset,
        processorId: typing.Union[MetaOapg.properties.processorId, None, str, uuid.UUID, schemas.Unset] = schemas.unset,
        organizationId: typing.Union[MetaOapg.properties.organizationId, str, uuid.UUID, schemas.Unset] = schemas.unset,
        templateDescription: typing.Union[MetaOapg.properties.templateDescription, str, schemas.Unset] = schemas.unset,
        heuristicFn: typing.Union[MetaOapg.properties.heuristicFn, None, str, schemas.Unset] = schemas.unset,
        heuristicFnLanguage: typing.Union[MetaOapg.properties.heuristicFnLanguage, str, schemas.Unset] = schemas.unset,
        aiPromptFormat: typing.Union[MetaOapg.properties.aiPromptFormat, None, str, schemas.Unset] = schemas.unset,
        aiImageUrls: typing.Union[MetaOapg.properties.aiImageUrls, list, tuple, schemas.Unset] = schemas.unset,
        humanPrompt: typing.Union[MetaOapg.properties.humanPrompt, None, str, schemas.Unset] = schemas.unset,
        classifierValuePath: typing.Union[MetaOapg.properties.classifierValuePath, None, str, schemas.Unset] = schemas.unset,
        classifierExpectedValuePath: typing.Union[MetaOapg.properties.classifierExpectedValuePath, None, str, schemas.Unset] = schemas.unset,
        multiClassOptions: typing.Union[MetaOapg.properties.multiClassOptions, list, tuple, schemas.Unset] = schemas.unset,
        prodEvalActive: typing.Union[MetaOapg.properties.prodEvalActive, bool, schemas.Unset] = schemas.unset,
        samplingProbability: typing.Union[MetaOapg.properties.samplingProbability, None, decimal.Decimal, int, float, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'EvaluatorV2':
        return super().__new__(
            cls,
            *_args,
            createdAt=createdAt,
            valueType=valueType,
            name=name,
            id=id,
            runCondition=runCondition,
            updatedAt=updatedAt,
            who=who,
            archivedAt=archivedAt,
            icon=icon,
            options=options,
            aiModel=aiModel,
            pipelineId=pipelineId,
            processorId=processorId,
            organizationId=organizationId,
            templateDescription=templateDescription,
            heuristicFn=heuristicFn,
            heuristicFnLanguage=heuristicFnLanguage,
            aiPromptFormat=aiPromptFormat,
            aiImageUrls=aiImageUrls,
            humanPrompt=humanPrompt,
            classifierValuePath=classifierValuePath,
            classifierExpectedValuePath=classifierExpectedValuePath,
            multiClassOptions=multiClassOptions,
            prodEvalActive=prodEvalActive,
            samplingProbability=samplingProbability,
            _configuration=_configuration,
            **kwargs,
        )

from gentrace.model.unix_seconds_nullable import UnixSecondsNullable



# coding: utf-8

"""


    Generated by: https://openapi-generator.tech
"""

from dataclasses import dataclass
import typing_extensions
import urllib3
from urllib3._collections import HTTPHeaderDict

from gentrace import api_client, exceptions
from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401

from gentrace.model.run_path_section import RunPathSection

# Path params
IdSchema = schemas.UUIDSchema
RequestRequiredPathParams = typing_extensions.TypedDict(
    'RequestRequiredPathParams',
    {
        'id': typing.Union[IdSchema, str, uuid.UUID, ],
    }
)
RequestOptionalPathParams = typing_extensions.TypedDict(
    'RequestOptionalPathParams',
    {
    },
    total=False
)


class RequestPathParams(RequestRequiredPathParams, RequestOptionalPathParams):
    pass


request_path_id = api_client.PathParameter(
    name="id",
    style=api_client.ParameterStyle.SIMPLE,
    schema=IdSchema,
    required=True,
)
# body param


class SchemaForRequestBodyApplicationJson(
    schemas.DictSchema
):


    class MetaOapg:
        
        class properties:
            
            
            class testRuns(
                schemas.ListSchema
            ):
            
            
                class MetaOapg:
                    
                    
                    class items(
                        schemas.DictSchema
                    ):
                    
                    
                        class MetaOapg:
                            required = {
                                "inputs",
                                "caseId",
                            }
                            
                            class properties:
                                
                                
                                class id(
                                    schemas.UUIDBase,
                                    schemas.StrBase,
                                    schemas.NoneBase,
                                    schemas.Schema,
                                    schemas.NoneStrMixin
                                ):
                                
                                
                                    class MetaOapg:
                                        format = 'uuid'
                                
                                
                                    def __new__(
                                        cls,
                                        *_args: typing.Union[None, str, uuid.UUID, ],
                                        _configuration: typing.Optional[schemas.Configuration] = None,
                                    ) -> 'id':
                                        return super().__new__(
                                            cls,
                                            *_args,
                                            _configuration=_configuration,
                                        )
                                caseId = schemas.UUIDSchema
                                
                                
                                class inputs(
                                    schemas.DictSchema
                                ):
                                
                                
                                    class MetaOapg:
                                        additional_properties = schemas.AnyTypeSchema
                                    
                                    def __getitem__(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                                        # dict_instance[name] accessor
                                        return super().__getitem__(name)
                                    
                                    def get_item_oapg(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                                        return super().get_item_oapg(name)
                                
                                    def __new__(
                                        cls,
                                        *_args: typing.Union[dict, frozendict.frozendict, ],
                                        _configuration: typing.Optional[schemas.Configuration] = None,
                                        **kwargs: typing.Union[MetaOapg.additional_properties, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                                    ) -> 'inputs':
                                        return super().__new__(
                                            cls,
                                            *_args,
                                            _configuration=_configuration,
                                            **kwargs,
                                        )
                                
                                
                                class outputs(
                                    schemas.DictSchema
                                ):
                                
                                
                                    class MetaOapg:
                                        additional_properties = schemas.AnyTypeSchema
                                    
                                    def __getitem__(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                                        # dict_instance[name] accessor
                                        return super().__getitem__(name)
                                    
                                    def get_item_oapg(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                                        return super().get_item_oapg(name)
                                
                                    def __new__(
                                        cls,
                                        *_args: typing.Union[dict, frozendict.frozendict, ],
                                        _configuration: typing.Optional[schemas.Configuration] = None,
                                        **kwargs: typing.Union[MetaOapg.additional_properties, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                                    ) -> 'outputs':
                                        return super().__new__(
                                            cls,
                                            *_args,
                                            _configuration=_configuration,
                                            **kwargs,
                                        )
                                
                                
                                class path(
                                    schemas.ListBase,
                                    schemas.NoneBase,
                                    schemas.Schema,
                                    schemas.NoneTupleMixin
                                ):
                                
                                
                                    class MetaOapg:
                                        
                                        @staticmethod
                                        def items() -> typing.Type['RunPathSection']:
                                            return RunPathSection
                                
                                
                                    def __new__(
                                        cls,
                                        *_args: typing.Union[list, tuple, None, ],
                                        _configuration: typing.Optional[schemas.Configuration] = None,
                                    ) -> 'path':
                                        return super().__new__(
                                            cls,
                                            *_args,
                                            _configuration=_configuration,
                                        )
                                
                                
                                class error(
                                    schemas.StrBase,
                                    schemas.NoneBase,
                                    schemas.Schema,
                                    schemas.NoneStrMixin
                                ):
                                
                                
                                    def __new__(
                                        cls,
                                        *_args: typing.Union[None, str, ],
                                        _configuration: typing.Optional[schemas.Configuration] = None,
                                    ) -> 'error':
                                        return super().__new__(
                                            cls,
                                            *_args,
                                            _configuration=_configuration,
                                        )
                                __annotations__ = {
                                    "id": id,
                                    "caseId": caseId,
                                    "inputs": inputs,
                                    "outputs": outputs,
                                    "path": path,
                                    "error": error,
                                }
                        
                        inputs: MetaOapg.properties.inputs
                        caseId: MetaOapg.properties.caseId
                        
                        @typing.overload
                        def __getitem__(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
                        
                        @typing.overload
                        def __getitem__(self, name: typing_extensions.Literal["caseId"]) -> MetaOapg.properties.caseId: ...
                        
                        @typing.overload
                        def __getitem__(self, name: typing_extensions.Literal["inputs"]) -> MetaOapg.properties.inputs: ...
                        
                        @typing.overload
                        def __getitem__(self, name: typing_extensions.Literal["outputs"]) -> MetaOapg.properties.outputs: ...
                        
                        @typing.overload
                        def __getitem__(self, name: typing_extensions.Literal["path"]) -> MetaOapg.properties.path: ...
                        
                        @typing.overload
                        def __getitem__(self, name: typing_extensions.Literal["error"]) -> MetaOapg.properties.error: ...
                        
                        @typing.overload
                        def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
                        
                        def __getitem__(self, name: typing.Union[typing_extensions.Literal["id", "caseId", "inputs", "outputs", "path", "error", ], str]):
                            # dict_instance[name] accessor
                            return super().__getitem__(name)
                        
                        
                        @typing.overload
                        def get_item_oapg(self, name: typing_extensions.Literal["id"]) -> typing.Union[MetaOapg.properties.id, schemas.Unset]: ...
                        
                        @typing.overload
                        def get_item_oapg(self, name: typing_extensions.Literal["caseId"]) -> MetaOapg.properties.caseId: ...
                        
                        @typing.overload
                        def get_item_oapg(self, name: typing_extensions.Literal["inputs"]) -> MetaOapg.properties.inputs: ...
                        
                        @typing.overload
                        def get_item_oapg(self, name: typing_extensions.Literal["outputs"]) -> typing.Union[MetaOapg.properties.outputs, schemas.Unset]: ...
                        
                        @typing.overload
                        def get_item_oapg(self, name: typing_extensions.Literal["path"]) -> typing.Union[MetaOapg.properties.path, schemas.Unset]: ...
                        
                        @typing.overload
                        def get_item_oapg(self, name: typing_extensions.Literal["error"]) -> typing.Union[MetaOapg.properties.error, schemas.Unset]: ...
                        
                        @typing.overload
                        def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
                        
                        def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["id", "caseId", "inputs", "outputs", "path", "error", ], str]):
                            return super().get_item_oapg(name)
                        
                    
                        def __new__(
                            cls,
                            *_args: typing.Union[dict, frozendict.frozendict, ],
                            inputs: typing.Union[MetaOapg.properties.inputs, dict, frozendict.frozendict, ],
                            caseId: typing.Union[MetaOapg.properties.caseId, str, uuid.UUID, ],
                            id: typing.Union[MetaOapg.properties.id, None, str, uuid.UUID, schemas.Unset] = schemas.unset,
                            outputs: typing.Union[MetaOapg.properties.outputs, dict, frozendict.frozendict, schemas.Unset] = schemas.unset,
                            path: typing.Union[MetaOapg.properties.path, list, tuple, None, schemas.Unset] = schemas.unset,
                            error: typing.Union[MetaOapg.properties.error, None, str, schemas.Unset] = schemas.unset,
                            _configuration: typing.Optional[schemas.Configuration] = None,
                            **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                        ) -> 'items':
                            return super().__new__(
                                cls,
                                *_args,
                                inputs=inputs,
                                caseId=caseId,
                                id=id,
                                outputs=outputs,
                                path=path,
                                error=error,
                                _configuration=_configuration,
                                **kwargs,
                            )
            
                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple[typing.Union[MetaOapg.items, dict, frozendict.frozendict, ]], typing.List[typing.Union[MetaOapg.items, dict, frozendict.frozendict, ]]],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'testRuns':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )
            
                def __getitem__(self, i: int) -> MetaOapg.items:
                    return super().__getitem__(i)
            __annotations__ = {
                "testRuns": testRuns,
            }
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["testRuns"]) -> MetaOapg.properties.testRuns: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["testRuns", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["testRuns"]) -> typing.Union[MetaOapg.properties.testRuns, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["testRuns", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        testRuns: typing.Union[MetaOapg.properties.testRuns, list, tuple, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'SchemaForRequestBodyApplicationJson':
        return super().__new__(
            cls,
            *_args,
            testRuns=testRuns,
            _configuration=_configuration,
            **kwargs,
        )


request_body_any_type = api_client.RequestBody(
    content={
        'application/json': api_client.MediaType(
            schema=SchemaForRequestBodyApplicationJson),
    },
    required=True,
)


class SchemaFor200ResponseBodyApplicationJson(
    schemas.DictSchema
):


    class MetaOapg:
        required = {
            "resultId",
        }
        
        class properties:
            resultId = schemas.UUIDSchema
            __annotations__ = {
                "resultId": resultId,
            }
    
    resultId: MetaOapg.properties.resultId
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["resultId"]) -> MetaOapg.properties.resultId: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["resultId", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["resultId"]) -> MetaOapg.properties.resultId: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["resultId", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        resultId: typing.Union[MetaOapg.properties.resultId, str, uuid.UUID, ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'SchemaFor200ResponseBodyApplicationJson':
        return super().__new__(
            cls,
            *_args,
            resultId=resultId,
            _configuration=_configuration,
            **kwargs,
        )


class SchemaFor200ResponseBodyApplicationJsonCharsetutf8(
    schemas.DictSchema
):


    class MetaOapg:
        required = {
            "resultId",
        }
        
        class properties:
            resultId = schemas.UUIDSchema
            __annotations__ = {
                "resultId": resultId,
            }
    
    resultId: MetaOapg.properties.resultId
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["resultId"]) -> MetaOapg.properties.resultId: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["resultId", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["resultId"]) -> MetaOapg.properties.resultId: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["resultId", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        resultId: typing.Union[MetaOapg.properties.resultId, str, uuid.UUID, ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'SchemaFor200ResponseBodyApplicationJsonCharsetutf8':
        return super().__new__(
            cls,
            *_args,
            resultId=resultId,
            _configuration=_configuration,
            **kwargs,
        )


@dataclass
class ApiResponseFor200(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: typing.Union[
        SchemaFor200ResponseBodyApplicationJson,
        SchemaFor200ResponseBodyApplicationJsonCharsetutf8,
    ]
    headers: schemas.Unset = schemas.unset


_response_for_200 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor200,
    content={
        'application/json': api_client.MediaType(
            schema=SchemaFor200ResponseBodyApplicationJson),
        'application/json; charset=utf-8': api_client.MediaType(
            schema=SchemaFor200ResponseBodyApplicationJsonCharsetutf8),
    },
)


@dataclass
class ApiResponseFor400(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: schemas.Unset = schemas.unset
    headers: schemas.Unset = schemas.unset


_response_for_400 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor400,
)


@dataclass
class ApiResponseFor404(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: schemas.Unset = schemas.unset
    headers: schemas.Unset = schemas.unset


_response_for_404 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor404,
)


@dataclass
class ApiResponseFor500(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: schemas.Unset = schemas.unset
    headers: schemas.Unset = schemas.unset


_response_for_500 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor500,
)
_all_accept_content_types = (
    'application/json',
    'application/json; charset=utf-8',
)


class BaseApi(api_client.Api):
    @typing.overload
    def _v1_test_result_simple_id_post_oapg(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: typing_extensions.Literal["application/json"] = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
        ApiResponseFor200,
    ]: ...

    @typing.overload
    def _v1_test_result_simple_id_post_oapg(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
        ApiResponseFor200,
    ]: ...


    @typing.overload
    def _v1_test_result_simple_id_post_oapg(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        skip_deserialization: typing_extensions.Literal[True],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
    ) -> api_client.ApiResponseWithoutDeserialization: ...

    @typing.overload
    def _v1_test_result_simple_id_post_oapg(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = ...,
    ) -> typing.Union[
        ApiResponseFor200,
        api_client.ApiResponseWithoutDeserialization,
    ]: ...

    def _v1_test_result_simple_id_post_oapg(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = 'application/json',
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ):
        """
        Add more test runs to a result (simple version)
        :param skip_deserialization: If true then api_response.response will be set but
            api_response.body and api_response.headers will not be deserialized into schema
            class instances
        """
        self._verify_typed_dict_inputs_oapg(RequestPathParams, path_params)
        used_path = path.value

        _path_params = {}
        for parameter in (
            request_path_id,
        ):
            parameter_data = path_params.get(parameter.name, schemas.unset)
            if parameter_data is schemas.unset:
                continue
            serialized_data = parameter.serialize(parameter_data)
            _path_params.update(serialized_data)

        for k, v in _path_params.items():
            used_path = used_path.replace('{%s}' % k, v)

        _headers = HTTPHeaderDict()
        # TODO add cookie handling
        if accept_content_types:
            for accept_content_type in accept_content_types:
                _headers.add('Accept', accept_content_type)

        if body is schemas.unset:
            raise exceptions.ApiValueError(
                'The required body parameter has an invalid value of: unset. Set a valid value instead')
        _fields = None
        _body = None
        serialized_data = request_body_any_type.serialize(body, content_type)
        _headers.add('Content-Type', content_type)
        if 'fields' in serialized_data:
            _fields = serialized_data['fields']
        elif 'body' in serialized_data:
            _body = serialized_data['body']
        response = self.api_client.call_api(
            resource_path=used_path,
            method='post'.upper(),
            headers=_headers,
            fields=_fields,
            body=_body,
            auth_settings=_auth,
            stream=stream,
            timeout=timeout,
        )

        if skip_deserialization:
            api_response = api_client.ApiResponseWithoutDeserialization(response=response)
        else:
            response_for_status = _status_code_to_response.get(str(response.status))
            if response_for_status:
                api_response = response_for_status.deserialize(response, self.api_client.configuration)
            else:
                api_response = api_client.ApiResponseWithoutDeserialization(response=response)

        if not 200 <= response.status <= 299:
            raise exceptions.ApiException(
                status=response.status,
                reason=response.reason,
                api_response=api_response
            )

        return api_response


class V1TestResultSimpleIdPost(BaseApi):
    # this class is used by api classes that refer to endpoints with operationId fn names

    @typing.overload
    def v1_test_result_simple_id_post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: typing_extensions.Literal["application/json"] = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
        ApiResponseFor200,
    ]: ...

    @typing.overload
    def v1_test_result_simple_id_post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
        ApiResponseFor200,
    ]: ...


    @typing.overload
    def v1_test_result_simple_id_post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        skip_deserialization: typing_extensions.Literal[True],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
    ) -> api_client.ApiResponseWithoutDeserialization: ...

    @typing.overload
    def v1_test_result_simple_id_post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = ...,
    ) -> typing.Union[
        ApiResponseFor200,
        api_client.ApiResponseWithoutDeserialization,
    ]: ...

    def v1_test_result_simple_id_post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = 'application/json',
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ):
        return self._v1_test_result_simple_id_post_oapg(
            body=body,
            path_params=path_params,
            content_type=content_type,
            accept_content_types=accept_content_types,
            stream=stream,
            timeout=timeout,
            skip_deserialization=skip_deserialization
        )


class ApiForpost(BaseApi):
    # this class is used by api classes that refer to endpoints by path and http method names

    @typing.overload
    def post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: typing_extensions.Literal["application/json"] = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
        ApiResponseFor200,
    ]: ...

    @typing.overload
    def post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
        ApiResponseFor200,
    ]: ...


    @typing.overload
    def post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        skip_deserialization: typing_extensions.Literal[True],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
    ) -> api_client.ApiResponseWithoutDeserialization: ...

    @typing.overload
    def post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = ...,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = ...,
    ) -> typing.Union[
        ApiResponseFor200,
        api_client.ApiResponseWithoutDeserialization,
    ]: ...

    def post(
        self,
        body: typing.Union[SchemaForRequestBodyApplicationJson,dict, frozendict.frozendict, ],
        content_type: str = 'application/json',
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ):
        return self._v1_test_result_simple_id_post_oapg(
            body=body,
            path_params=path_params,
            content_type=content_type,
            accept_content_types=accept_content_types,
            stream=stream,
            timeout=timeout,
            skip_deserialization=skip_deserialization
        )





# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class UpdateDatasetV2(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        
        class properties:
            
            
            class name(
                schemas.StrSchema
            ):
                pass
            
            
            class description(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'description':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            isGolden = schemas.BoolSchema
            isArchived = schemas.BoolSchema
            __annotations__ = {
                "name": name,
                "description": description,
                "isGolden": isGolden,
                "isArchived": isArchived,
            }
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["description"]) -> MetaOapg.properties.description: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["isGolden"]) -> MetaOapg.properties.isGolden: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["isArchived"]) -> MetaOapg.properties.isArchived: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["name", "description", "isGolden", "isArchived", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["name"]) -> typing.Union[MetaOapg.properties.name, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["description"]) -> typing.Union[MetaOapg.properties.description, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["isGolden"]) -> typing.Union[MetaOapg.properties.isGolden, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["isArchived"]) -> typing.Union[MetaOapg.properties.isArchived, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["name", "description", "isGolden", "isArchived", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        name: typing.Union[MetaOapg.properties.name, str, schemas.Unset] = schemas.unset,
        description: typing.Union[MetaOapg.properties.description, None, str, schemas.Unset] = schemas.unset,
        isGolden: typing.Union[MetaOapg.properties.isGolden, bool, schemas.Unset] = schemas.unset,
        isArchived: typing.Union[MetaOapg.properties.isArchived, bool, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'UpdateDatasetV2':
        return super().__new__(
            cls,
            *_args,
            name=name,
            description=description,
            isGolden=isGolden,
            isArchived=isArchived,
            _configuration=_configuration,
            **kwargs,
        )



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class TestEvaluation(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "billingGpt4OutputTokens",
            "createdAt",
            "billingGpt4InputTokens",
            "billingGpt35InputTokens",
            "billingGpt35OutputTokens",
            "id",
            "runId",
            "isPending",
            "evaluatorId",
            "updatedAt",
        }
        
        class properties:
            id = schemas.UUIDSchema
            createdAt = schemas.StrSchema
            updatedAt = schemas.StrSchema
            isPending = schemas.BoolSchema
            evaluatorId = schemas.UUIDSchema
            runId = schemas.UUIDSchema
            billingGpt4InputTokens = schemas.IntSchema
            billingGpt4OutputTokens = schemas.IntSchema
            billingGpt35InputTokens = schemas.IntSchema
            billingGpt35OutputTokens = schemas.IntSchema
            
            
            class debug(
                schemas.DictBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneFrozenDictMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'debug':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class evalLabel(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'evalLabel':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class evalValue(
                schemas.NumberBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneDecimalMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, decimal.Decimal, int, float, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'evalValue':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class manualCreatedByEmail(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'manualCreatedByEmail':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            __annotations__ = {
                "id": id,
                "createdAt": createdAt,
                "updatedAt": updatedAt,
                "isPending": isPending,
                "evaluatorId": evaluatorId,
                "runId": runId,
                "billingGpt4InputTokens": billingGpt4InputTokens,
                "billingGpt4OutputTokens": billingGpt4OutputTokens,
                "billingGpt35InputTokens": billingGpt35InputTokens,
                "billingGpt35OutputTokens": billingGpt35OutputTokens,
                "debug": debug,
                "evalLabel": evalLabel,
                "evalValue": evalValue,
                "manualCreatedByEmail": manualCreatedByEmail,
            }
    
    billingGpt4OutputTokens: MetaOapg.properties.billingGpt4OutputTokens
    createdAt: MetaOapg.properties.createdAt
    billingGpt4InputTokens: MetaOapg.properties.billingGpt4InputTokens
    billingGpt35InputTokens: MetaOapg.properties.billingGpt35InputTokens
    billingGpt35OutputTokens: MetaOapg.properties.billingGpt35OutputTokens
    id: MetaOapg.properties.id
    runId: MetaOapg.properties.runId
    isPending: MetaOapg.properties.isPending
    evaluatorId: MetaOapg.properties.evaluatorId
    updatedAt: MetaOapg.properties.updatedAt
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["createdAt"]) -> MetaOapg.properties.createdAt: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["updatedAt"]) -> MetaOapg.properties.updatedAt: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["isPending"]) -> MetaOapg.properties.isPending: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["evaluatorId"]) -> MetaOapg.properties.evaluatorId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["runId"]) -> MetaOapg.properties.runId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["billingGpt4InputTokens"]) -> MetaOapg.properties.billingGpt4InputTokens: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["billingGpt4OutputTokens"]) -> MetaOapg.properties.billingGpt4OutputTokens: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["billingGpt35InputTokens"]) -> MetaOapg.properties.billingGpt35InputTokens: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["billingGpt35OutputTokens"]) -> MetaOapg.properties.billingGpt35OutputTokens: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["debug"]) -> MetaOapg.properties.debug: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["evalLabel"]) -> MetaOapg.properties.evalLabel: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["evalValue"]) -> MetaOapg.properties.evalValue: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["manualCreatedByEmail"]) -> MetaOapg.properties.manualCreatedByEmail: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["id", "createdAt", "updatedAt", "isPending", "evaluatorId", "runId", "billingGpt4InputTokens", "billingGpt4OutputTokens", "billingGpt35InputTokens", "billingGpt35OutputTokens", "debug", "evalLabel", "evalValue", "manualCreatedByEmail", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["createdAt"]) -> MetaOapg.properties.createdAt: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["updatedAt"]) -> MetaOapg.properties.updatedAt: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["isPending"]) -> MetaOapg.properties.isPending: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["evaluatorId"]) -> MetaOapg.properties.evaluatorId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["runId"]) -> MetaOapg.properties.runId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["billingGpt4InputTokens"]) -> MetaOapg.properties.billingGpt4InputTokens: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["billingGpt4OutputTokens"]) -> MetaOapg.properties.billingGpt4OutputTokens: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["billingGpt35InputTokens"]) -> MetaOapg.properties.billingGpt35InputTokens: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["billingGpt35OutputTokens"]) -> MetaOapg.properties.billingGpt35OutputTokens: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["debug"]) -> typing.Union[MetaOapg.properties.debug, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["evalLabel"]) -> typing.Union[MetaOapg.properties.evalLabel, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["evalValue"]) -> typing.Union[MetaOapg.properties.evalValue, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["manualCreatedByEmail"]) -> typing.Union[MetaOapg.properties.manualCreatedByEmail, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["id", "createdAt", "updatedAt", "isPending", "evaluatorId", "runId", "billingGpt4InputTokens", "billingGpt4OutputTokens", "billingGpt35InputTokens", "billingGpt35OutputTokens", "debug", "evalLabel", "evalValue", "manualCreatedByEmail", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        billingGpt4OutputTokens: typing.Union[MetaOapg.properties.billingGpt4OutputTokens, decimal.Decimal, int, ],
        createdAt: typing.Union[MetaOapg.properties.createdAt, str, ],
        billingGpt4InputTokens: typing.Union[MetaOapg.properties.billingGpt4InputTokens, decimal.Decimal, int, ],
        billingGpt35InputTokens: typing.Union[MetaOapg.properties.billingGpt35InputTokens, decimal.Decimal, int, ],
        billingGpt35OutputTokens: typing.Union[MetaOapg.properties.billingGpt35OutputTokens, decimal.Decimal, int, ],
        id: typing.Union[MetaOapg.properties.id, str, uuid.UUID, ],
        runId: typing.Union[MetaOapg.properties.runId, str, uuid.UUID, ],
        isPending: typing.Union[MetaOapg.properties.isPending, bool, ],
        evaluatorId: typing.Union[MetaOapg.properties.evaluatorId, str, uuid.UUID, ],
        updatedAt: typing.Union[MetaOapg.properties.updatedAt, str, ],
        debug: typing.Union[MetaOapg.properties.debug, dict, frozendict.frozendict, None, schemas.Unset] = schemas.unset,
        evalLabel: typing.Union[MetaOapg.properties.evalLabel, None, str, schemas.Unset] = schemas.unset,
        evalValue: typing.Union[MetaOapg.properties.evalValue, None, decimal.Decimal, int, float, schemas.Unset] = schemas.unset,
        manualCreatedByEmail: typing.Union[MetaOapg.properties.manualCreatedByEmail, None, str, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'TestEvaluation':
        return super().__new__(
            cls,
            *_args,
            billingGpt4OutputTokens=billingGpt4OutputTokens,
            createdAt=createdAt,
            billingGpt4InputTokens=billingGpt4InputTokens,
            billingGpt35InputTokens=billingGpt35InputTokens,
            billingGpt35OutputTokens=billingGpt35OutputTokens,
            id=id,
            runId=runId,
            isPending=isPending,
            evaluatorId=evaluatorId,
            updatedAt=updatedAt,
            debug=debug,
            evalLabel=evalLabel,
            evalValue=evalValue,
            manualCreatedByEmail=manualCreatedByEmail,
            _configuration=_configuration,
            **kwargs,
        )



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class FullRun(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "organizationId",
            "pipelineRunId",
            "startTime",
            "endTime",
            "pipelineId",
        }
        
        class properties:
            pipelineRunId = schemas.UUIDSchema
            pipelineId = schemas.UUIDSchema
            organizationId = schemas.UUIDSchema
            startTime = schemas.StrSchema
            endTime = schemas.StrSchema
            
            
            class cost(
                schemas.NumberBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneDecimalMixin
            ):
            
            
                class MetaOapg:
                    inclusive_minimum = 0
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, decimal.Decimal, int, float, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'cost':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class elapsed(
                schemas.NumberBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneDecimalMixin
            ):
            
            
                class MetaOapg:
                    inclusive_minimum = 0
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, decimal.Decimal, int, float, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'elapsed':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class feedback(
                schemas.NumberBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneDecimalMixin
            ):
            
            
                class MetaOapg:
                    inclusive_maximum = 1
                    inclusive_minimum = 0
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, decimal.Decimal, int, float, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'feedback':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class lastInvocation(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'lastInvocation':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class inputs(
                schemas.DictBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneFrozenDictMixin
            ):
            
            
                class MetaOapg:
                    additional_properties = schemas.AnyTypeSchema
            
                
                def __getitem__(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    # dict_instance[name] accessor
                    return super().__getitem__(name)
                
                def get_item_oapg(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    return super().get_item_oapg(name)
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[MetaOapg.additional_properties, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                ) -> 'inputs':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class outputs(
                schemas.DictBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneFrozenDictMixin
            ):
            
            
                class MetaOapg:
                    additional_properties = schemas.AnyTypeSchema
            
                
                def __getitem__(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    # dict_instance[name] accessor
                    return super().__getitem__(name)
                
                def get_item_oapg(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    return super().get_item_oapg(name)
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[MetaOapg.additional_properties, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                ) -> 'outputs':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class renderHTMLKey(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'renderHTMLKey':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class metadata(
                schemas.DictBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneFrozenDictMixin
            ):
            
            
                class MetaOapg:
                    
                    @staticmethod
                    def additional_properties() -> typing.Type['MetadataValueObject']:
                        return MetadataValueObject
            
                
                def __getitem__(self, name: typing.Union[str, ]) -> 'MetadataValueObject':
                    # dict_instance[name] accessor
                    return super().__getitem__(name)
                
                def get_item_oapg(self, name: typing.Union[str, ]) -> 'MetadataValueObject':
                    return super().get_item_oapg(name)
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: 'MetadataValueObject',
                ) -> 'metadata':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class path(
                schemas.ListBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneTupleMixin
            ):
            
            
                class MetaOapg:
                    
                    @staticmethod
                    def items() -> typing.Type['RunPathSection']:
                        return RunPathSection
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[list, tuple, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'path':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            
            
            class error(
                schemas.StrBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneStrMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[None, str, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'error':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                    )
            __annotations__ = {
                "pipelineRunId": pipelineRunId,
                "pipelineId": pipelineId,
                "organizationId": organizationId,
                "startTime": startTime,
                "endTime": endTime,
                "cost": cost,
                "elapsed": elapsed,
                "feedback": feedback,
                "lastInvocation": lastInvocation,
                "inputs": inputs,
                "outputs": outputs,
                "renderHTMLKey": renderHTMLKey,
                "metadata": metadata,
                "path": path,
                "error": error,
            }
    
    organizationId: MetaOapg.properties.organizationId
    pipelineRunId: MetaOapg.properties.pipelineRunId
    startTime: MetaOapg.properties.startTime
    endTime: MetaOapg.properties.endTime
    pipelineId: MetaOapg.properties.pipelineId
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["pipelineRunId"]) -> MetaOapg.properties.pipelineRunId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["pipelineId"]) -> MetaOapg.properties.pipelineId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["organizationId"]) -> MetaOapg.properties.organizationId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["startTime"]) -> MetaOapg.properties.startTime: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["endTime"]) -> MetaOapg.properties.endTime: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["cost"]) -> MetaOapg.properties.cost: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["elapsed"]) -> MetaOapg.properties.elapsed: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["feedback"]) -> MetaOapg.properties.feedback: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["lastInvocation"]) -> MetaOapg.properties.lastInvocation: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["inputs"]) -> MetaOapg.properties.inputs: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["outputs"]) -> MetaOapg.properties.outputs: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["renderHTMLKey"]) -> MetaOapg.properties.renderHTMLKey: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["metadata"]) -> MetaOapg.properties.metadata: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["path"]) -> MetaOapg.properties.path: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["error"]) -> MetaOapg.properties.error: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["pipelineRunId", "pipelineId", "organizationId", "startTime", "endTime", "cost", "elapsed", "feedback", "lastInvocation", "inputs", "outputs", "renderHTMLKey", "metadata", "path", "error", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["pipelineRunId"]) -> MetaOapg.properties.pipelineRunId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["pipelineId"]) -> MetaOapg.properties.pipelineId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["organizationId"]) -> MetaOapg.properties.organizationId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["startTime"]) -> MetaOapg.properties.startTime: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["endTime"]) -> MetaOapg.properties.endTime: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["cost"]) -> typing.Union[MetaOapg.properties.cost, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["elapsed"]) -> typing.Union[MetaOapg.properties.elapsed, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["feedback"]) -> typing.Union[MetaOapg.properties.feedback, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["lastInvocation"]) -> typing.Union[MetaOapg.properties.lastInvocation, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["inputs"]) -> typing.Union[MetaOapg.properties.inputs, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["outputs"]) -> typing.Union[MetaOapg.properties.outputs, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["renderHTMLKey"]) -> typing.Union[MetaOapg.properties.renderHTMLKey, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["metadata"]) -> typing.Union[MetaOapg.properties.metadata, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["path"]) -> typing.Union[MetaOapg.properties.path, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["error"]) -> typing.Union[MetaOapg.properties.error, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["pipelineRunId", "pipelineId", "organizationId", "startTime", "endTime", "cost", "elapsed", "feedback", "lastInvocation", "inputs", "outputs", "renderHTMLKey", "metadata", "path", "error", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        organizationId: typing.Union[MetaOapg.properties.organizationId, str, uuid.UUID, ],
        pipelineRunId: typing.Union[MetaOapg.properties.pipelineRunId, str, uuid.UUID, ],
        startTime: typing.Union[MetaOapg.properties.startTime, str, ],
        endTime: typing.Union[MetaOapg.properties.endTime, str, ],
        pipelineId: typing.Union[MetaOapg.properties.pipelineId, str, uuid.UUID, ],
        cost: typing.Union[MetaOapg.properties.cost, None, decimal.Decimal, int, float, schemas.Unset] = schemas.unset,
        elapsed: typing.Union[MetaOapg.properties.elapsed, None, decimal.Decimal, int, float, schemas.Unset] = schemas.unset,
        feedback: typing.Union[MetaOapg.properties.feedback, None, decimal.Decimal, int, float, schemas.Unset] = schemas.unset,
        lastInvocation: typing.Union[MetaOapg.properties.lastInvocation, None, str, schemas.Unset] = schemas.unset,
        inputs: typing.Union[MetaOapg.properties.inputs, dict, frozendict.frozendict, None, schemas.Unset] = schemas.unset,
        outputs: typing.Union[MetaOapg.properties.outputs, dict, frozendict.frozendict, None, schemas.Unset] = schemas.unset,
        renderHTMLKey: typing.Union[MetaOapg.properties.renderHTMLKey, None, str, schemas.Unset] = schemas.unset,
        metadata: typing.Union[MetaOapg.properties.metadata, dict, frozendict.frozendict, None, schemas.Unset] = schemas.unset,
        path: typing.Union[MetaOapg.properties.path, list, tuple, None, schemas.Unset] = schemas.unset,
        error: typing.Union[MetaOapg.properties.error, None, str, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'FullRun':
        return super().__new__(
            cls,
            *_args,
            organizationId=organizationId,
            pipelineRunId=pipelineRunId,
            startTime=startTime,
            endTime=endTime,
            pipelineId=pipelineId,
            cost=cost,
            elapsed=elapsed,
            feedback=feedback,
            lastInvocation=lastInvocation,
            inputs=inputs,
            outputs=outputs,
            renderHTMLKey=renderHTMLKey,
            metadata=metadata,
            path=path,
            error=error,
            _configuration=_configuration,
            **kwargs,
        )

from gentrace.model.metadata_value_object import MetadataValueObject
from gentrace.model.run_path_section import RunPathSection



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class CreateSingleTestCase(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "inputs",
            "name",
        }
        
        class properties:
            name = schemas.StrSchema
            
            
            class inputs(
                schemas.DictSchema
            ):
            
            
                class MetaOapg:
                    additional_properties = schemas.AnyTypeSchema
                
                def __getitem__(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    # dict_instance[name] accessor
                    return super().__getitem__(name)
                
                def get_item_oapg(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    return super().get_item_oapg(name)
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[MetaOapg.additional_properties, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                ) -> 'inputs':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class pipelineSlug(
                schemas.StrSchema
            ):
                pass
            
            
            class datasetId(
                schemas.StrSchema
            ):
                pass
            
            
            class expectedOutputs(
                schemas.DictBase,
                schemas.NoneBase,
                schemas.Schema,
                schemas.NoneFrozenDictMixin
            ):
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, None, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'expectedOutputs':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            __annotations__ = {
                "name": name,
                "inputs": inputs,
                "pipelineSlug": pipelineSlug,
                "datasetId": datasetId,
                "expectedOutputs": expectedOutputs,
            }
    
    inputs: MetaOapg.properties.inputs
    name: MetaOapg.properties.name
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["inputs"]) -> MetaOapg.properties.inputs: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["pipelineSlug"]) -> MetaOapg.properties.pipelineSlug: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["datasetId"]) -> MetaOapg.properties.datasetId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["expectedOutputs"]) -> MetaOapg.properties.expectedOutputs: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["name", "inputs", "pipelineSlug", "datasetId", "expectedOutputs", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["inputs"]) -> MetaOapg.properties.inputs: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["pipelineSlug"]) -> typing.Union[MetaOapg.properties.pipelineSlug, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["datasetId"]) -> typing.Union[MetaOapg.properties.datasetId, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["expectedOutputs"]) -> typing.Union[MetaOapg.properties.expectedOutputs, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["name", "inputs", "pipelineSlug", "datasetId", "expectedOutputs", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        inputs: typing.Union[MetaOapg.properties.inputs, dict, frozendict.frozendict, ],
        name: typing.Union[MetaOapg.properties.name, str, ],
        pipelineSlug: typing.Union[MetaOapg.properties.pipelineSlug, str, schemas.Unset] = schemas.unset,
        datasetId: typing.Union[MetaOapg.properties.datasetId, str, schemas.Unset] = schemas.unset,
        expectedOutputs: typing.Union[MetaOapg.properties.expectedOutputs, dict, frozendict.frozendict, None, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'CreateSingleTestCase':
        return super().__new__(
            cls,
            *_args,
            inputs=inputs,
            name=name,
            pipelineSlug=pipelineSlug,
            datasetId=datasetId,
            expectedOutputs=expectedOutputs,
            _configuration=_configuration,
            **kwargs,
        )



# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class CreateEvaluationV2(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "runId",
            "evaluatorId",
        }
        
        class properties:
            evaluatorId = schemas.UUIDSchema
            runId = schemas.UUIDSchema
            note = schemas.StrSchema
            evalLabel = schemas.StrSchema
            evalValue = schemas.NumberSchema
            __annotations__ = {
                "evaluatorId": evaluatorId,
                "runId": runId,
                "note": note,
                "evalLabel": evalLabel,
                "evalValue": evalValue,
            }
    
    runId: MetaOapg.properties.runId
    evaluatorId: MetaOapg.properties.evaluatorId
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["evaluatorId"]) -> MetaOapg.properties.evaluatorId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["runId"]) -> MetaOapg.properties.runId: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["note"]) -> MetaOapg.properties.note: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["evalLabel"]) -> MetaOapg.properties.evalLabel: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["evalValue"]) -> MetaOapg.properties.evalValue: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["evaluatorId", "runId", "note", "evalLabel", "evalValue", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["evaluatorId"]) -> MetaOapg.properties.evaluatorId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["runId"]) -> MetaOapg.properties.runId: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["note"]) -> typing.Union[MetaOapg.properties.note, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["evalLabel"]) -> typing.Union[MetaOapg.properties.evalLabel, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["evalValue"]) -> typing.Union[MetaOapg.properties.evalValue, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["evaluatorId", "runId", "note", "evalLabel", "evalValue", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        runId: typing.Union[MetaOapg.properties.runId, str, uuid.UUID, ],
        evaluatorId: typing.Union[MetaOapg.properties.evaluatorId, str, uuid.UUID, ],
        note: typing.Union[MetaOapg.properties.note, str, schemas.Unset] = schemas.unset,
        evalLabel: typing.Union[MetaOapg.properties.evalLabel, str, schemas.Unset] = schemas.unset,
        evalValue: typing.Union[MetaOapg.properties.evalValue, decimal.Decimal, int, float, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'CreateEvaluationV2':
        return super().__new__(
            cls,
            *_args,
            runId=runId,
            evaluatorId=evaluatorId,
            note=note,
            evalLabel=evalLabel,
            evalValue=evalValue,
            _configuration=_configuration,
            **kwargs,
        )
